#include "kamod_pca9533.h"
#include "kamami_l496_conn.h"

/* Initialization of PCA9533 LED dimmer. 
 * The function should be called before using any other PCA9533 function. 
 * It checks if communication with the sensor is possible.
 * @return 0 on succes, non-zero value if i2c error */
int32_t pca9533_init()
{
	uint8_t tmp = 0;
	
	conn_i2c_init(CONN_I2C_KAMOD);
	
	if (conn_i2c_read(CONN_I2C_KAMOD, PCA9533_ADDR, PCA9533_CTRL_REG_INPUT, 1, &tmp, 1) != 0)
		return -1;
	else
		return 0;
}

/* Setting of PWM 0 prescaler value. 
 * @param prescaler: prescaler value. 
 * @return */
void pca9533_set_psc_pwm0(uint8_t prescaler)
{
	conn_i2c_write(CONN_I2C_KAMOD, PCA9533_ADDR, PCA9533_CTRL_REG_PSC0, 1, &prescaler, 1);
}	
/* Setting of PWM 0 frequency value. 
 * @param frequency: frequency value. 
 * @return */
void pca9533_set_freq_pwm0(uint32_t frequency)
{
	uint8_t val;
	
	val = 152/frequency - 1;
	
	conn_i2c_write(CONN_I2C_KAMOD, PCA9533_ADDR, PCA9533_CTRL_REG_PSC0, 1, &val, 1);
}
/* Setting of PWM 0 duty cycle value. 
 * @param duty: duty cycle value. 
 * @return */
void pca9533_set_duty_pwm0(uint8_t duty)
{
	conn_i2c_write(CONN_I2C_KAMOD, PCA9533_ADDR, PCA9533_CTRL_REG_PWM0, 1, &duty, 1);
}
/* Setting of PWM 1 prescaler value. 
 * @param prescaler: prescaler value. 
 * @return */
void pca9533_set_psc_pwm1(uint8_t prescaler)
{
	conn_i2c_write(CONN_I2C_KAMOD, PCA9533_ADDR, PCA9533_CTRL_REG_PSC1, 1, &prescaler, 1);
}
/* Setting of PWM 1 frequency value. 
 * @param frequency: frequency value. 
 * @return */
void pca9533_set_freq_pwm1(uint32_t frequency)
{
	uint8_t val;
	
	val = 152/frequency - 1;
	
	conn_i2c_write(CONN_I2C_KAMOD, PCA9533_ADDR, PCA9533_CTRL_REG_PSC1, 1, &val, 1);
}
	/* Setting of PWM 1 duty cycle value. 
 * @param duty: duty cycle value. 
 * @return */
void pca9533_set_duty_pwm1(uint8_t duty)
{
	conn_i2c_write(CONN_I2C_KAMOD, PCA9533_ADDR, PCA9533_CTRL_REG_PWM1, 1, &duty, 1);
}
	/* Setting of PWM 0 parameters value. 
 * @param frequency: frequency value. 
 * @param brightness: brightness value (0-100 %). 
 * @return */
void pca9533_set_pwm0(uint32_t frequency, uint8_t brightness)
{
	uint32_t duty = brightness * 255/100;
	
	pca9533_set_freq_pwm0(frequency);
	pca9533_set_duty_pwm0((uint8_t)duty);
}
	/* Setting of PWM 1 parameters value. 
 * @param frequency: frequency value. 
 * @param brightness: brightness value (0-100 %). 
 * @return */
void pca9533_set_pwm1(uint32_t frequency, uint8_t brightness)
{
	uint32_t duty = brightness * 255/100;
	
	pca9533_set_freq_pwm1(frequency);
	pca9533_set_duty_pwm1((uint8_t)duty);
}
	/* Setting of LED mode.
 * In fuction is changed mode of led channel, other led channel are blinking without changes.
 * @param led: led value [PCA9533_LED_GREEN, PCA9533_LED_RED, PCA9533_LED_BLUE]. 
 * @param mode: mode value [PCA9533_MODE_OFF, PCA9533_MODE_ON, PCA9533_MODE_PWM0, PCA9533_MODE_PWM1]. 
 * @return 0 on success, non-zero if error i2c*/
int32_t pca9533_set_led(uint8_t led, uint8_t mode)
{
	uint8_t tmp = 0;
	uint8_t reg_val;
	
	if (conn_i2c_read(CONN_I2C_KAMOD, PCA9533_ADDR, PCA9533_CTRL_REG_LS0, 1, &tmp, 1) != 0)
		return -1;

	tmp &= ~(3 << (led * 2));
	reg_val = tmp | (mode << (led * 2));
	
	conn_i2c_write(CONN_I2C_KAMOD, PCA9533_ADDR, PCA9533_CTRL_REG_LS0, 1, &reg_val, 1);
	
	return 0;
}
